From 76ea7d068e580adc5a10affc80d9be12c52b1577 Mon Sep 17 00:00:00 2001
From: Andrew Price <andrewprice@andrewalexanderprice.com>
Date: Mon, 5 Sep 2011 10:35:12 +0930
Subject: [PATCH 8/8] support code to sync two images to a single frame

---
 include/vlc_picture.h               |    7 +++++++
 modules/video_filter/stereoscopy.c  |   10 +++++-----
 modules/video_output/msw/direct3d.c |    7 ++++++-
 3 files changed, 18 insertions(+), 6 deletions(-)

diff --git a/include/vlc_picture.h b/include/vlc_picture.h
index 5387e38..40143e8 100644
--- a/include/vlc_picture.h
+++ b/include/vlc_picture.h
@@ -61,6 +61,13 @@ typedef struct plane_t
  */
 typedef struct picture_release_sys_t picture_release_sys_t;
 
+/* A bit flag for i_eye that indicates to wait for the next frame before
+   presenting. */
+#define STEREO_WAIT_FOR_NEXT_FRAME_BIT (1 << 8)
+
+/* Mask the lower bits of i_eye to return the eye that should be drawn. */
+#define STEREO_EYE_MASK 0xFF
+
 /**
  * Video picture
  */
diff --git a/modules/video_filter/stereoscopy.c b/modules/video_filter/stereoscopy.c
index 56a8645..b9402cd 100644
--- a/modules/video_filter/stereoscopy.c
+++ b/modules/video_filter/stereoscopy.c
@@ -904,9 +904,7 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_inpic )
             picture_Release( p_inpic );
             return NULL;
     }
-
-    /* CORRECT METHOD BUT COMMENTED OUT WHILE 'DROPPING PICTURES' ISSUE REMAINS */
-
+	
 	/* calculate time difference and find mid point (to insert other frame at) */
 	mtime_t i_currentTime = p_inpic->date;
 	mtime_t i_midPoint = p_sys->i_lastTime + (i_currentTime - p_sys->i_lastTime) / 2;
@@ -922,7 +920,8 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_inpic )
         return NULL;
     }
 
-    p_leftOut->i_eye = 1;
+	/* left eye and tell vout to wait for the next eye instead of immediately presenting */
+    p_leftOut->i_eye = 1 | STEREO_WAIT_FOR_NEXT_FRAME_BIT;
 
     p_leftOut->p_next = p_rightOut = DecodeImageYUV( p_filter, p_inpic,
         p_sys->i_rightEyeMethod, chromaformat);
@@ -943,7 +942,8 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_inpic )
     
 
     /* DECODES LEFT AND RIGHT FRAMES ALTERNATIVELY, DROPS A FRAME BUT IS A
-       WORKAROUND UNTIL DROPPING PICTURES IS FIXED */
+       WORKAROUND UNTIL DROPPING PICTURES IS FIXED - fixed :)
+	   left in here incase I have to switch back to it for some reason */
 	/*
     if(p_sys->b_leftEyeLast)
     {
diff --git a/modules/video_output/msw/direct3d.c b/modules/video_output/msw/direct3d.c
index 49bc47b..bd1846c 100644
--- a/modules/video_output/msw/direct3d.c
+++ b/modules/video_output/msw/direct3d.c
@@ -321,7 +321,10 @@ static void Prepare(vout_display_t *vd, picture_t *picture, subpicture_t *subpic
     }
 
     d3d_region_t picture_region;
-    if (!Direct3DImportPicture(vd, &picture_region, surface, i_eye)) {
+    if (!Direct3DImportPicture(vd, &picture_region, surface, i_eye)
+       /*!(i_eye & STEREO_WAIT_FOR_NEXT_FRAME_BIT)*/ /* causes it to flicker,
+                                                         perhapes stopping Display
+                                                         will fix it */) {
         int subpicture_region_count     = 0;
         d3d_region_t *subpicture_region = NULL;
         if (subpicture)
@@ -1406,6 +1409,8 @@ static int Direct3DImportPicture(vout_display_t *vd,
         return VLC_EGENERIC;
     }
 
+	/* mask out all flags so i_eye only stores the eye number */
+	i_eye &= STEREO_EYE_MASK;
 
     /* msg_Dbg(vd, "Receiving eye (i_eye=%i)", i_eye); */
 
-- 
1.7.4.msysgit.0

